==> eyflutter_uikit-0.0.8%2B8.tar.gz: lib/uikit/state/elements_state.dart <==
import 'dart:collection';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:provider/provider.dart';

typedef Widget ConsumerElementBuilder<M extends ElementModel>(BuildContext context, M model);
typedef Widget ConsumerItemBuilder<M extends ElementModel, I>(BuildContext context, M model, int position, I item);

class ElementModel<P> extends CusChangeNotifier {
  late P _parent;
  dynamic _source;

  P get parent => _parent;

  ElementModel(P parent) {
    this._parent = parent;
  }

  //设置数据源
  void setSource<D>(D? data) {
    _source = data;
  }
}

abstract class ElementListModel<P, ITEM> extends ElementModel<P> {
  bool isNoMore = false;
  bool isEmptyData = false;
  int _page = 1;

  /// 获取当前分页索引
  int get pageIndex => _page;

  /// get model listener
  OnBaseModelListener get modelListener;

  //刷新组件controller
  CusRefreshController? refreshController;

  /// 数据集合
  final List<ITEM> _items = [];

  /// 禁止外部改变数据集
  UnmodifiableListView<ITEM> get items => UnmodifiableListView(_items);

  /// 获取数据类目
  ITEM getItem(int index) => items[index];

  /// 统计记录数
  int get count => _items.length;

  ElementListModel(P parent) : super(parent);

  /// 重置页面属性
  void reset() {
    _page = 1;
    refreshController?.isAutoTriggerLoad = true;
  }

  /// 将item从集合中移除
  /// item 要移除的数据
  void removeItem({required ITEM item}) {
    if (item == null || _items.length == 0) {
      return;
    }
    _items.remove(item);
  }

  void updateAt(int index, ITEM item) {
    if (item == null || index < 0 || (index + 1) >= _items.length) {
      return;
    }
    _items[index] = item;
  }

  /// 将item添加到集合中
  /// [items] 数据集合
  /// [isClear] 添加数据至集合前是否需要清空
  void addAll({required List<ITEM> items, bool isClear = false}) {
    if (items.isNotEmptyList) {
      if (isClear) {
        _items.clear();
      }
      _items.addAll(items);
    }
  }

  /// 将item添加到集合中
  /// [item] 数据模型
  void add({required ITEM item}) {
    if (item != null) {
      _items.add(item);
    }
  }

  /// 插件数据至集合中
  /// [item] 数据模型
  /// [index] 插入数据至集合中指定位置的索引,null或大于items.length将添加至集合末尾。
  void insert({required ITEM item, int? index}) {
    if (item != null) {
      var len = _items.length;
      var idx = (index == null || (index + 1) >= len) ? len : index;
      _items.insert(idx, item);
    }
  }

  /// 清除数据
  void clear() {
    _items.clear();
  }

  /// 设置空视图隐藏与显示
  void showEmptyView(bool show) {
    isEmptyData = show;
    notifierLazy();
  }

  /// 分页接口请求完成后执行，配置相关状态
  /// [response] 分页数据对象,请求失败时不需要传入
  void finishRequest({Map<String, dynamic>? response}) {
    var firstFieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response?[firstFieldName] ?? false;
    if ((isFirstPage is! bool) || isFirstPage) {
      this.clear();
    } else {
      _page++;
    }
    var nextFieldName = DioManager.instance.configInterceptor?.hasNextPageKey() ?? "";
    var hasNextPage = response?[nextFieldName] ?? false;
    if ((hasNextPage is! bool) || !hasNextPage) {
      isNoMore = true;
    } else {
      isNoMore = false;
    }
    var countFieldName = DioManager.instance.configInterceptor?.countKey() ?? "";
    var count = response?[countFieldName] ?? 0;
    if (_page == 1 && isNoMore && count == 0) {
      isEmptyData = true;
    } else {
      isEmptyData = false;
    }
  }

  /// 是否首页数据
  bool isFirstPage(Map<String, dynamic> response) {
    return StateUtils.instance.isFirstPage(response);
  }

  void finishRefresh() {
    refreshController?.finishRefresh(success: true, noMore: isNoMore);
  }

  void finishLoad() {
    Future.delayed(Duration(milliseconds: 200), () => refreshController?.finishLoad(success: true, noMore: isNoMore));
  }

  /// 初始化数据
  void onInitData(BuildContext context) {
    refreshController?.isAutoTriggerLoad = true;
    modelListener.onRequestData(context, RequestWay.refresh);
  }

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        _page = 1;
        isNoMore = false;
        refreshController?.isAutoTriggerLoad = true;
        modelListener.onRequestData(context, RequestWay.refresh);
      } else {
        finishRefresh();
      }
    });
  }

  /// 加载数据
  void load(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        modelListener.onRequestData(context, RequestWay.load);
      } else {
        finishLoad();
      }
    });
  }
}

class _StateWidget {
  _StateWidget._();

  static Widget subscribeNotifierWidget<M extends ElementModel>(M model, ConsumerElementBuilder<M> childBuilder) {
    return ChangeNotifierProvider<M>(
      create: (_) => model,
      child: Consumer<M>(
        builder: (context, m, _) {
          return childBuilder(context, m);
        },
      ),
    );
  }
}

abstract class ElementBuilder<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  D? _source;

  D? get source => _getSource();

  D? _getSource() {
    var mSource = _model._source;
    if (mSource is D) {
      _source = mSource;
    }
    return _source;
  }

  void setSource(D? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  /// 构建初始化
  void onBuildInit() {}

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

abstract class ElementListBuilder<M extends ElementListModel, D> extends ElementBuilder<M, D> {
  /// 背景
  Color get backgroundColor => Colors.white;

  /// 背景图片
  String get backgroundImage => "";

  /// 背景对齐方式
  AlignmentGeometry get backgroundAlignment => Alignment.bottomCenter;

  /// 容器边框圆角
  BorderRadiusGeometry get borderRadius => BorderRadius.zero;

  /// 内边距
  EdgeInsetsGeometry get padding => EdgeInsets.zero;

  /// 启用刷新
  bool get enableRefresh => true;

  /// 启用加载
  bool get enableLoad => true;

  /// 构建页面顶部视图
  Widget onBuildHeadView() {
    return Container();
  }

  /// 渲染视图bottom对象
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  /// 空视图对象
  Widget onEmptyRenderObject() {
    return Container();
  }

  /// 渲染视图top对象
  void onRenderTopObject(List<Widget> list) {}

  /// 构建渲染对象
  void onRenderObject(List<Widget> list) {}

  @override
  Widget builder(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<M>(model, (context, model) {
      return Container(
        alignment: model.isEmptyData ? Alignment.center : null,
        padding: backgroundImage.isEmptyString ? padding : null,
        decoration: BoxDecoration(
          color: backgroundColor,
          borderRadius: borderRadius,
        ),
        child: _contentWidget(context),
      );
    });
  }

  Widget _contentWidget(BuildContext context) {
    if (backgroundImage.isEmptyString) {
      return Column(
        children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
      );
    } else {
      return Stack(
        children: [
          Container(
            padding: padding,
            decoration: BoxDecoration(
              color: backgroundColor,
              image: DecorationImage(
                image: AssetImageProvider.render(backgroundImage, isProjectImage: !PluginUtils.instance.isRunPlugin),
                alignment: backgroundAlignment,
                fit: BoxFit.fitWidth,
              ),
              borderRadius: borderRadius,
            ),
          ),
          Column(
            children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
          )
        ],
      );
    }
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: _bindListWidgets(),
        ),
        loadIndicatorWidget: Container(
          width: 86,
          alignment: Alignment.center,
          child: model.isNoMore
              ? Text(
                  RefreshUtils.instance.refreshConfig?.noMoreText() ?? "",
                  style: TextStyle(color: Color(0xb3333333), fontSize: 12),
                )
              : RefreshUtils.instance.refreshConfig?.loadingWidget(),
        ),
        enableRefresh: enableRefresh,
        enableLoad: enableLoad,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        },
        onLoad: (EasyRefreshController controller) {
          model.load(context);
        });
  }

  List<Widget> _bindListWidgets() {
    List<Widget> lst = [];
    onRenderTopObject(lst);
    if (model.isEmptyData) {
      lst.add(onEmptyRenderObject());
    } else {
      onRenderObject(lst);
    }
    return lst;
  }
}

abstract class ElementsBuilder<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  List<I>? _source;

  List<I>? get source => _source;

  void setSource(List<I>? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  Widget itemBuilder(BuildContext context, int position, I item);

  void itemProcessing(I item);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

/// 元素状态
class ElementState<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  ElementState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidget({required BuildContext context, D? source, required ElementBuilder<M, D> builder}) {
    if (source != null) {
      builder.setSource(source);
    }
    builder.setModel(_model);
    builder.onBuildInit();
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}

/// 元素状态
class ElementsState<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  ElementsState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件集转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidgets({required BuildContext context, List<I>? source, required ElementsBuilder<M, I> builder}) {
    builder.setSource(source);
    builder.setModel(this._model);
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}

==> eyflutter_uikit-0.1.4%2B8.tar.gz: lib/uikit/state/elements_state.dart <==
import 'dart:collection';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:provider/provider.dart';

typedef Widget ConsumerElementBuilder<M extends ElementModel>(BuildContext context, M model);
typedef Widget ConsumerItemBuilder<M extends ElementModel, I>(BuildContext context, M model, int position, I item);

class ElementModel<P> extends CusChangeNotifier {
  late P _parent;
  dynamic _source;

  P get parent => _parent;

  ElementModel(P parent) {
    this._parent = parent;
  }

  //设置数据源
  void setSource<D>(D? data) {
    _source = data;
  }
}

abstract class ElementListModel<P, ITEM> extends ElementModel<P> {
  bool isNoMore = false;
  bool isEmptyData = false;
  int _page = 1;
  int _total = 0;

  /// 获取当前分页索引
  int get pageIndex => _page;

  /// get model listener
  OnBaseModelListener get modelListener;

  //刷新组件controller
  CusRefreshController? refreshController;

  /// 数据集合
  final List<ITEM> _items = [];

  /// 禁止外部改变数据集
  UnmodifiableListView<ITEM> get items => UnmodifiableListView(_items);

  /// 获取数据类目
  ITEM getItem(int index) => items[index];

  /// 统计记录数
  int get count => _items.length;

  ElementListModel(P parent) : super(parent);

  /// 重置页面属性
  void reset() {
    _page = 1;
    refreshController?.isAutoTriggerLoad = true;
  }

  /// 将item从集合中移除
  /// item 要移除的数据
  void removeItem({required ITEM item}) {
    if (item == null || _items.length == 0) {
      return;
    }
    _items.remove(item);
  }

  void updateAt(int index, ITEM item) {
    if (item == null || index < 0 || (index + 1) >= _items.length) {
      return;
    }
    _items[index] = item;
  }

  /// 将item添加到集合中
  /// [items] 数据集合
  /// [isClear] 添加数据至集合前是否需要清空
  void addAll({required List<ITEM> items, bool isClear = false}) {
    if (items.isNotEmptyList) {
      if (isClear) {
        _items.clear();
      }
      _items.addAll(items);
    }
  }

  /// 将item添加到集合中
  /// [item] 数据模型
  void add({required ITEM item}) {
    if (item != null) {
      _items.add(item);
    }
  }

  /// 插件数据至集合中
  /// [item] 数据模型
  /// [index] 插入数据至集合中指定位置的索引,null或大于items.length将添加至集合末尾。
  void insert({required ITEM item, int? index}) {
    if (item != null) {
      var len = _items.length;
      var idx = (index == null || (index + 1) >= len) ? len : index;
      _items.insert(idx, item);
    }
  }

  /// 清除数据
  void clear() {
    _items.clear();
  }

  /// 设置空视图隐藏与显示
  void showEmptyView(bool show) {
    isEmptyData = show;
    notifierLazy();
  }

  /// 分页接口请求完成后执行，配置相关状态
  /// [response] 分页数据对象,请求失败时不需要传入
  void finishRequest({Map<String, dynamic>? response}) {
    var nextFieldName = DioManager.instance.configInterceptor?.hasNextPageKey() ?? "";
    var hasNextPage = response?[nextFieldName] ?? false;
    if ((hasNextPage is! bool) || !hasNextPage) {
      isNoMore = true;
    } else {
      isNoMore = false;
    }
    var countFieldName = DioManager.instance.configInterceptor?.countKey() ?? "";
    _total = response?[countFieldName] ?? 0;
    if (_page == 1 && isNoMore && _total == 0) {
      isEmptyData = true;
    } else {
      isEmptyData = false;
    }
    var firstFieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response?[firstFieldName] ?? false;
    if ((isFirstPage is! bool) || isFirstPage) {
      this.clear();
    } else if ((hasNextPage is bool) && hasNextPage) {
      _page++;
    }
  }

  /// 获取总记录数
  int get total => _total;

  /// 是否首页数据
  bool isFirstPage(Map<String, dynamic> response) {
    return StateUtils.instance.isFirstPage(response);
  }

  void finishRefresh() {
    refreshController?.finishRefresh(success: true, noMore: isNoMore);
  }

  void finishLoad() {
    Future.delayed(Duration(milliseconds: 200), () => refreshController?.finishLoad(success: true, noMore: isNoMore));
  }

  /// 初始化数据
  void onInitData(BuildContext context) {
    refreshController?.isAutoTriggerLoad = true;
    modelListener.onRequestData(context, RequestWay.refresh);
  }

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        _page = 1;
        isNoMore = false;
        refreshController?.isAutoTriggerLoad = true;
        modelListener.onRequestData(context, RequestWay.refresh);
      } else {
        finishRefresh();
      }
    });
  }

  /// 加载数据
  void load(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        modelListener.onRequestData(context, RequestWay.load);
      } else {
        finishLoad();
      }
    });
  }
}

class _StateWidget {
  _StateWidget._();

  static Widget subscribeNotifierWidget<M extends ElementModel>(M model, ConsumerElementBuilder<M> childBuilder) {
    return ChangeNotifierProvider<M>(
      create: (_) => model,
      child: Consumer<M>(
        builder: (context, m, _) {
          return childBuilder(context, m);
        },
      ),
    );
  }
}

abstract class ElementBuilder<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  D? _source;

  D? get source => _getSource();

  D? _getSource() {
    var mSource = _model._source;
    if (mSource is D) {
      _source = mSource;
    }
    return _source;
  }

  void setSource(D? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  /// 构建初始化
  void onBuildInit() {}

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

abstract class ElementListBuilder<M extends ElementListModel, D> extends ElementBuilder<M, D> {
  /// 背景
  Color get backgroundColor => Colors.white;

  /// 背景图片
  String get backgroundImage => "";

  /// 背景对齐方式
  AlignmentGeometry get backgroundAlignment => Alignment.bottomCenter;

  /// 容器边框圆角
  BorderRadiusGeometry get borderRadius => BorderRadius.zero;

  /// 内边距
  EdgeInsetsGeometry get padding => EdgeInsets.zero;

  /// 启用刷新
  bool get enableRefresh => true;

  /// 启用加载
  bool get enableLoad => true;

  /// 构建页面顶部视图
  Widget onBuildHeadView() {
    return Container();
  }

  /// 渲染视图bottom对象
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  /// 空视图对象
  Widget onEmptyRenderObject() {
    return Container();
  }

  /// 渲染视图top对象
  void onRenderTopObject(List<Widget> list) {}

  /// 构建渲染对象
  void onRenderObject(List<Widget> list) {}

  @override
  Widget builder(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<M>(model, (context, model) {
      return Container(
        alignment: model.isEmptyData ? Alignment.center : null,
        padding: backgroundImage.isEmptyString ? padding : null,
        decoration: BoxDecoration(
          color: backgroundColor,
          borderRadius: borderRadius,
        ),
        child: _contentWidget(context),
      );
    });
  }

  Widget _contentWidget(BuildContext context) {
    if (backgroundImage.isEmptyString) {
      return Column(
        children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
      );
    } else {
      return Stack(
        children: [
          Container(
            padding: padding,
            decoration: BoxDecoration(
              color: backgroundColor,
              image: DecorationImage(
                image: AssetImageProvider.render(backgroundImage, isProjectImage: !PluginUtils.instance.isRunPlugin),
                alignment: backgroundAlignment,
                fit: BoxFit.fitWidth,
              ),
              borderRadius: borderRadius,
            ),
          ),
          Column(
            children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
          )
        ],
      );
    }
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: _bindListWidgets(),
        ),
        loadIndicatorWidget: Container(
          width: 86,
          alignment: Alignment.center,
          child: model.isNoMore
              ? Text(
                  RefreshUtils.instance.refreshConfig?.noMoreText() ?? "",
                  style: TextStyle(color: Color(0xb3333333), fontSize: 12),
                )
              : RefreshUtils.instance.refreshConfig?.loadingWidget(),
        ),
        enableRefresh: enableRefresh,
        enableLoad: enableLoad,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        },
        onLoad: (EasyRefreshController controller) {
          model.load(context);
        });
  }

  List<Widget> _bindListWidgets() {
    List<Widget> lst = [];
    onRenderTopObject(lst);
    if (model.isEmptyData) {
      lst.add(onEmptyRenderObject());
    } else {
      onRenderObject(lst);
    }
    return lst;
  }
}

abstract class ElementsBuilder<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  List<I>? _source;

  List<I>? get source => _source;

  void setSource(List<I>? data) {
    this._source = data;
  }

  void setModel(M model) {
    this._model = model;
  }

  void onBuildComplete() {}

  Widget builder(BuildContext context);

  Widget itemBuilder(BuildContext context, int position, I item);

  void itemProcessing(I item);

  /// 订阅消费组件
  /// [model] 订阅模型
  /// [builder] 组件构建器
  Widget subscribeNotifierWidget(M model, ConsumerElementBuilder<M> builder) {
    _model = model;
    return _StateWidget.subscribeNotifierWidget<M>(model, builder);
  }
}

/// 元素状态
class ElementState<M extends ElementModel, D> {
  late M _model;

  M get model => _model;

  ElementState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidget({required BuildContext context, D? source, required ElementBuilder<M, D> builder}) {
    if (source != null) {
      builder.setSource(source);
    }
    builder.setModel(_model);
    builder.onBuildInit();
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}

/// 元素状态
class ElementsState<M extends ElementModel, I> {
  late M _model;

  M get model => _model;

  ElementsState({required M model}) {
    this._model = model;
  }

  /// 消费对象组件集转换
  /// [context] 视图上下文
  /// [source] 原数据
  /// [builder] 对象消费者构建器
  Widget convertToWidgets({required BuildContext context, List<I>? source, required ElementsBuilder<M, I> builder}) {
    builder.setSource(source);
    builder.setModel(this._model);
    var widget = builder.builder(context);
    builder.onBuildComplete();
    return widget;
  }
}
